package rayTrace;
import support.AsciiMatrixFile;


/** Describes a simple collection of elements, intersection is tested on all internal objects and then the first to be hit is returned.
 * Also the superclass of collection objects like Box 
 */
public class Collection extends Element {

	public Element elems[];
	
	public Collection(Element elements[]) {
		this.elems = elements;
	}
	
	public double boundingSphereRadius(){ throw new RuntimeException("boundingSphereRadius() not generally implmented for Collection yet"); }		
	
	
	@Override
	public Intersection findIntersection(Ray ray) {
		
		Intersection hitSides[] = new Intersection[elems.length];
		
		int firstHitIdx = -1;
		
		//get the intersection of this ray with every surface, and find the one we hit first
		for(int i=0;i<elems.length;i++){
			hitSides[i] = elems[i].findIntersection(ray);
			
			if(hitSides[i] == null)
				continue;
			
			if(firstHitIdx <0 || hitSides[i].distAlongRay < hitSides[firstHitIdx].distAlongRay)
				firstHitIdx = i;
		}
	
		if(firstHitIdx < 0)return null; //never hit any of the sides
		
		return hitSides[firstHitIdx];

	}
	
	public static Collection loadDiscs(String fileName, double reHitTolerence){
		double data[][] = AsciiMatrixFile.mustLoad(fileName,false);
		
		Element elems[] = new Element[data.length];
		
		for(int i=0;i<data.length;i++){			
			elems[i] = new Disc(new double[]{ data[i][0], data[i][1], data[i][2] },
								normaliseVector(new double[]{ data[i][3], data[i][4], data[i][5] }),
								data[i][6] / 2.0,
								(data[i][7] != 0) ? Element.MATTYPE_TRANSPARENT : Element.MATTYPE_REFLECTING,
								1,1,
								reHitTolerence );
		}
		
		return new Collection(elems);
		
	}

	public static Collection loadDishes(String fileName, double reHitTolerence){
		double data[][] = AsciiMatrixFile.mustLoad(fileName,false);
		
		Element elems[] = new Element[data.length];
		
		for(int i=0;i<data.length;i++){			
			elems[i] = new Dish(new double[]{ data[i][0], data[i][1], data[i][2] },
								normaliseVector(new double[]{ data[i][3], data[i][4], data[i][5] }),
								data[i][7],
								data[i][6],
								Element.MATTYPE_REFLECTING,
								1,1,
								reHitTolerence );
		}
		
		return new Collection(elems);
	}
	
	public void rotate(double point[], double matrix[][]){
		for(int i=0;i<elems.length;i++)
			elems[i].rotate(point, matrix);
	}
	
	public void shift(double dX[]){
		for(int i=0;i<elems.length;i++)
			elems[i].shift(dX);
	}
	
	public void addElement(Element newElem){
		Element newElems[] = new Element[elems.length + 1];
		System.arraycopy(elems,0,newElems,0,elems.length);
		newElems[elems.length] = newElem;
		elems = newElems;
	}


}
