package rayTrace;



/** An infinite plane, this is also the superclass of elements which are finite planes, like Disc and Square */
public class Plane extends Element {
	
	protected double normal[]; //normal vector
	protected double reHitTolerence;
	
	public Plane(double centre[], double normal[], int matType, double frontRIndex, double backRIndex, double reHitTolerence){
		this.centre =  centre;
		this.normal = normal;
		this.reHitTolerence = reHitTolerence;
		this.rIndexRatio = frontRIndex / backRIndex;
		this.matType = matType;
	}
	
	public double boundingSphereRadius(){ return Double.POSITIVE_INFINITY; } 
	
	
	public void shift(double[] dX) {
		for(int i=0;i<3;i++)
			centre[i] += dX[i];
	}
	
	public void rotate(double point[], double matrix[][]){
		for(int i=0;i<3;i++)
			centre[i] -= point[i];
			
		double newCentre[] = new double[3];
		double newNormal[] = new double[3];
		
		for(int i=0;i<3;i++)
			for(int j=0;j<3;j++){
				newCentre[i] += matrix[i][j] * centre[j];
				newNormal[i] += matrix[i][j] * normal[j];
			}
				
		for(int i=0;i<3;i++)
			centre[i] = point[i] + newCentre[i];
		
		normal = newNormal;
	}
	
	@Override
	public Intersection findIntersection(Ray ray) {
		double sum = 0;
		
		Intersection hit = new Intersection();
		
		for(int i=0;i<3;i++)
			sum += ray.unit[i] * normal[i];
		
		if(sum == 0)   // if // to plane, it never hits
			return null;	
		else if(sum < 0){ //if we hit the front of the plane
			for(int i=0;i<3;i++)
				hit.normal[i] = normal[i];
			hit.rIndexRatio = rIndexRatio;
		}else{ //if we hit the back of the plane
			for(int i=0;i<3;i++)
				hit.normal[i] = -normal[i]; //invert the normal
			hit.rIndexRatio = 1.0 / rIndexRatio;
		}
		
		double dist = 0;
							
		for(int i=0;i<3;i++)
			dist += (centre[i] - ray.start[i] ) * normal[i];
		
		dist /= sum;
		
		if(dist < reHitTolerence)
			return null;
				
		for(int i=0;i<3;i++)
			hit.pos[i] = ray.start[i] + dist * ray.unit[i];
		
		hit.distAlongRay = dist;
		hit.object = this;
		
		
		
		return hit;
	}
	
	/*//test for intersection of a ray and an unbounded plane
	function [xPos,dist] = planeIntersect(ray, pCentre, pNorm)
		if sum( ray.unit .* pNorm ) == 0 	//line is parallel with plane
			dist = -1;
			return;
		end
		dist = sum( (pCentre - ray.start) .*  pNorm ) / sum( ray.unit .* pNorm );
		xPos = ray.start + dist * ray.unit;
	endfunction */


}
